@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2020 Edgewall Software
+# Copyright (C) 2011-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -24,9 +24,11 @@ time during install.
 from html.parser import HTMLParser
 import io
 import os
+import pkg_resources
 import re
 from tokenize import generate_tokens, COMMENT, NAME, OP, STRING
 
+import jinja2
 from jinja2.ext import babel_extract as jinja2_extractor
 
 from distutils import log as distlog
@@ -36,8 +38,12 @@ from distutils.errors import DistutilsOptionError
 from setuptools.command.install_lib import install_lib as _install_lib
 
 
+_jinja2_ext_with = pkg_resources.parse_version(jinja2.__version__) < \
+                   pkg_resources.parse_version('3')
+
+
 def simplify_message(message):
-    """Transforms an extracted messsage (string or tuple) into one in
+    """Transforms an extracted message (string or tuple) into one in
     which the repeated white-space has been simplified to a single
     space.
 
@@ -143,7 +149,7 @@ try:
         if 'cleandoc_keywords' in options:
             cleandoc_keywords.update(options['cleandoc_keywords'])
 
-        tokens = generate_tokens(fileobj.readline)
+        tokens = generate_tokens(lambda: fileobj.readline().decode(encoding))
         tok = value = None
         for _ in tokens:
             prev_tok, prev_value = tok, value
@@ -166,7 +172,7 @@ try:
                 continue
             elif call_stack == -1 and tok == COMMENT:
                 # Strip the comment token from the line
-                value = value.decode(encoding)[1:].strip()
+                value = value[1:].strip()
                 if in_translator_comments and \
                         translator_comments[-1][0] == lineno - 1:
                     # We're already inside a translator comment, continue
@@ -206,7 +212,7 @@ try:
                         messages = tuple(messages)
                     else:
                         messages = messages[0]
-                    # Comments don't apply unless they immediately preceed the
+                    # Comments don't apply unless they immediately precede the
                     # message
                     if translator_comments and \
                             translator_comments[-1][0] < message_lineno - 1:
@@ -274,7 +280,8 @@ try:
         extractor = ScriptExtractor(out)
         extractor.feed(str(fileobj.read(), 'utf-8'))
         extractor.close()
-        out.seek(0)
+        # extract_javascript expects a binary file object
+        out = io.BytesIO(out.getvalue().encode('utf-8'))
         return extract_javascript(out, keywords, comment_tags, options)
 
 
@@ -292,6 +299,9 @@ try:
         """
         if fileobj:
             extractor = jinja2_extractor
+            options.setdefault('extensions', 'jinja2.ext.do, jinja2.ext.with_'
+                                             if _jinja2_ext_with else
+                                             'jinja2.ext.do')
             fileobj.seek(0)
             for m in extractor(fileobj, keywords, comment_tags, options):
                 # lineno, func, message, comments = m
@@ -470,9 +480,13 @@ try:
         from xml.etree import ElementTree
 
         def count_tags(text):
-            text = '<html>\n%s\n</html>' % text.encode('utf-8')
+            buf = io.StringIO()
+            buf.write('<html>\n')
+            buf.write(text)
+            buf.write('</html>')
+            buf.seek(0, 0)
             counts = {}
-            for event in ElementTree.iterparse(io.BytesIO(text)):
+            for event in ElementTree.iterparse(buf):
                 tag = event[1].tag
                 counts.setdefault(tag, 0)
                 counts[tag] += 1
