@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2023 Edgewall Software
 # Copyright (C) 2006-2007 Alec Thomas <alec@swapoff.org>
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -195,7 +195,7 @@ class Resource(object):
                 version = None
             if parent is False:
                 parent = None
-        resource = super(Resource, cls).__new__(cls)
+        resource = super().__new__(cls)
         resource.realm = realm
         resource.id = id
         resource.version = version
