@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2021 Edgewall Software
+# Copyright (C) 2006-2023 Edgewall Software
 # Copyright (C) 2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -21,16 +21,18 @@ from datetime import datetime
 from math import ceil
 import re
 
-from jinja2 import Markup, Undefined, contextfilter, evalcontextfilter
+from jinja2 import Undefined
 from jinja2.filters import make_attrgetter
 try:
-    from jinja2.utils import soft_unicode
-except:
-    from markupsafe import soft_str as soft_unicode
+    from jinja2 import pass_context, pass_eval_context
+except ImportError:
+    from jinja2 import (contextfilter as pass_context,
+                        evalcontextfilter as pass_eval_context)
 
 from trac.core import TracError
 from .datefmt import to_utimestamp, utc
-from .html import Fragment, classes, html_attribute, styles, tag
+from .html import (Fragment, Markup, classes, html_attribute, soft_unicode,
+                   styles, tag)
 from .text import javascript_quote
 
 __all__ = ['captioned_button', 'classes', 'first_last', 'group', 'istext',
@@ -75,7 +77,7 @@ def jinja2_update(jenv):
 
 # -- Jinja2 custom filters
 
-@evalcontextfilter
+@pass_eval_context
 def htmlattr_filter(_eval_ctx, d, autospace=True):
     """Create an SGML/XML attribute string based on the items in a dict.
 
@@ -101,8 +103,6 @@ def htmlattr_filter(_eval_ctx, d, autospace=True):
               'style': {'border-radius': '3px' if rounded,
                         'background': '#f7f7f7'}
              }|htmlattr}>
-        ...
-        </ul>
 
     Results in something like this:
 
@@ -110,8 +110,6 @@ def htmlattr_filter(_eval_ctx, d, autospace=True):
 
         <ul class="my list" id="list-42" checked="checked" autocomplete="on"
             style="border-radius: 3px; background: #f7f7f7">
-        ...
-        </ul>
 
     As you can see it automatically prepends a space in front of the item
     if the filter returned something unless the second parameter is false.
@@ -262,7 +260,7 @@ def group(iterable, num, predicate=None):
         yield tuple(buf)
 
 
-@contextfilter
+@pass_context
 def groupattr_filter(_eval_ctx, iterable, num, attr, *args, **kwargs):
     """Similar to `group`, but as an attribute filter."""
     attr_getter = make_attrgetter(_eval_ctx.environment, attr)
