@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -107,7 +107,7 @@ Interactive Trac administration console.
 Copyright (C) %(year)s Edgewall Software
 
 Type:  '?' or 'help' for help on commands.
-        """, version=TRAC_VERSION, year='2003-2020'))
+        """, version=TRAC_VERSION, year='2003-2023'))
         self.cmdloop()
 
     # Environment methods
@@ -233,9 +233,9 @@ Type:  '?' or 'help' for help on commands.
             if self.needs_upgrade is None:
                 self.needs_upgrade = self.__env.needs_upgrade()
         except TracError as e:
-            raise AdminCommandError(to_unicode(e))
+            raise AdminCommandError(to_unicode(e)) from e
         except Exception as e:
-            raise AdminCommandError(exception_to_unicode(e))
+            raise AdminCommandError(exception_to_unicode(e)) from e
         args = self.arg_tokenize(line)
         if args[0] == 'upgrade':
             self.needs_upgrade = None
