@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2023 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -73,8 +73,7 @@ class ConfigurationError(TracError):
     def __init__(self, message=None, title=None, show_traceback=False):
         if message is None:
             message = _("Look in the Trac log for more information.")
-        super(ConfigurationError, self).__init__(message, title,
-                                                 show_traceback)
+        super().__init__(message, title, show_traceback)
 
 
 class UnicodeConfigParser(ConfigParser):
@@ -92,7 +91,6 @@ class UnicodeConfigParser(ConfigParser):
     def __init__(self, ignorecase_option=True, **kwargs):
         self._ignorecase_option = ignorecase_option
         kwargs.setdefault('interpolation', None)
-        kwargs.setdefault('strict', False)
         ConfigParser.__init__(self, **kwargs)
 
     def optionxform(self, option):
@@ -189,7 +187,7 @@ class Configuration(object):
         """Return the specified option as boolean value.
 
         If the value of the option is one of "yes", "true", "enabled", "on",
-        or "1", this method wll return `True`, otherwise `False`.
+        or "1", this method will return `True`, otherwise `False`.
 
         Valid default input is a string or a bool. Returns a bool.
         """
@@ -360,7 +358,7 @@ class Configuration(object):
                     raise TracError(_("Error reading '%(file)s', make sure "
                                       "it is readable.", file=self.filename))
             except ParsingError as e:
-                raise TracError(e)
+                raise TracError(e) from e
             self._lastmtime = modtime
             self._pristine_parser = copy.deepcopy(self.parser)
             changed = True
