@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -49,7 +49,7 @@ class TracError(TracBaseError):
         above the error message.
         """
         from trac.util.translation import gettext
-        super(TracError, self).__init__(message)
+        super().__init__(message)
         self._message = message
         self.title = title or gettext(self.title)
         self.show_traceback = show_traceback
