@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2021 Edgewall Software
+# Copyright (C) 2005-2023 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -34,7 +34,7 @@ _like_escape_re = re.compile(r'([/_%])')
 _glob_escape_re = re.compile(r'[*?\[]')
 
 try:
-    import pysqlite2.dbapi2 as sqlite
+    import pysqlite3.dbapi2 as sqlite
 except ImportError:
     import sqlite3 as sqlite
 
@@ -45,10 +45,12 @@ pysqlite_version_string = get_pkginfo(sqlite).get('version',
                                                   '%d.%d.%s'
                                                   % pysqlite_version)
 min_sqlite_version = (3, 0, 0)
-min_pysqlite_version = (2, 6, 0)  # version provided by Python 2.7
 
 
 class PyFormatCursor(sqlite.Cursor):
+
+    __slots__ = ['cnx']
+
     def _rollback_on_error(self, function, *args, **kwargs):
         try:
             return function(self, *args, **kwargs)
@@ -170,9 +172,6 @@ class SQLiteConnector(Component):
                            "version is %(min_version)s.",
                            version=sqlite_version_string,
                            min_version='%d.%d.%d' % min_sqlite_version)
-        elif pysqlite_version < min_pysqlite_version:
-            self.error = _("Need at least PySqlite %(version)s or higher",
-                           version='%d.%d.%d' % min_pysqlite_version)
         yield 'sqlite', -1 if self.error else 1
 
     def get_connection(self, path, log=None, params={}):
@@ -211,16 +210,20 @@ class SQLiteConnector(Component):
             # this direct connect will create the database if needed
             cnx = sqlite.connect(path, isolation_level=None,
                                  timeout=int(params.get('timeout', 10000)))
-            with closing(cnx.cursor()) as cursor:
-                _set_journal_mode(cursor, params.get('journal_mode'))
-                set_synchronous(cursor, params.get('synchronous'))
-                insert_schema(cursor, schema)
-            cnx.isolation_level = 'DEFERRED'
+            try:
+                with closing(cnx.cursor()) as cursor:
+                    _set_journal_mode(cursor, params.get('journal_mode'))
+                    set_synchronous(cursor, params.get('synchronous'))
+                    insert_schema(cursor, schema)
+                cnx.isolation_level = 'DEFERRED'
+                cnx.commit()
+            finally:
+                cnx.close()
         else:
             cnx = self.get_connection(path, log, params)
             with closing(cnx.cursor()) as cursor:
                 insert_schema(cursor, schema)
-        cnx.commit()
+            cnx.commit()
 
     def destroy_db(self, path, log=None, params={}):
         if path != ':memory:':
@@ -448,7 +451,9 @@ class SQLiteConnection(ConnectionBase, ConnectionWrapper):
             column = row[1]
             type_ = row[2]
             pk = row[5]
-            if pk == 1 and type_ == 'integer':
+            # PRAGMA table_info() returns 'INTEGER' instead of 'integer'
+            # since SQLite 3.37. See https://www.sqlite.org/stricttables.html
+            if pk == 1 and type_.upper() == 'INTEGER':
                 key = [column]
                 auto_increment = True
             else:
