@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2023 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -76,7 +76,7 @@ class PatchRenderer(Component):
         """
         def _markup_intraline_change(fromlines, tolines):
             from trac.versioncontrol.diff import get_change_extent
-            for i in xrange(len(fromlines)):
+            for i in range(len(fromlines)):
                 fr, to = fromlines[i], tolines[i]
                 (start, end) = get_change_extent(fr, to)
                 if start != 0 or end != 0:
@@ -276,7 +276,7 @@ class PatchRenderer(Component):
                             b['type'] = 'rem'
                         elif len(f) == len(t):
                             _markup_intraline_change(f, t)
-                    for i in xrange(len(f)):
+                    for i in range(len(f)):
                         line = expandtabs(f[i], tabwidth, '\0\1')
                         line = escape(line, quotes=False)
                         line = '<del>'.join(space_re.sub(htmlify, seg)
@@ -285,7 +285,7 @@ class PatchRenderer(Component):
                         f[i] = Markup(line)
                         if 'meta' in base and i in base['meta']:
                             f[i] = Markup('<em>%s</em>') % f[i]
-                    for i in xrange(len(t)):
+                    for i in range(len(t)):
                         line = expandtabs(t[i], tabwidth, '\0\1')
                         line = escape(line, quotes=False)
                         line = '<ins>'.join(space_re.sub(htmlify, seg)
