@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2023 Edgewall Software
 # Copyright (C) 2006 Alec Thomas
 # Copyright (C) 2007 Eli Carter
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
@@ -651,7 +651,6 @@ class WorkflowMacro(WikiMacroBase):
                     raise ProcessorError("Invalid argument(s).")
 
             if file:
-                print(file)
                 text = RepositoryManager(self.env).read_file_by_path(file)
                 if text is None:
                     raise ProcessorError(
@@ -669,9 +668,9 @@ class WorkflowMacro(WikiMacroBase):
                 parser.readfp(io.StringIO(text))
             except ParsingError as e:
                 if is_macro:
-                    raise MacroError(exception_to_unicode(e))
+                    raise MacroError(exception_to_unicode(e)) from e
                 else:
-                    raise ProcessorError(exception_to_unicode(e))
+                    raise ProcessorError(exception_to_unicode(e)) from e
             raw_actions = list(parser.items('ticket-workflow'))
         actions = parse_workflow_config(raw_actions)
         states = list(
