@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2023 Edgewall Software
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -14,26 +14,31 @@
 # history and logs, available at https://trac.edgewall.org/log/.
 
 import errno
+import hmac
 import os
 import subprocess
 import time
 
 
-# Windows doesn't have a crypt module by default.
 try:
-    from crypt import crypt
+    from passlib.context import CryptContext
 except ImportError:
+    # Windows doesn't have a crypt module by default.
     try:
-        from passlib.hash import des_crypt
+        from crypt import crypt
     except ImportError:
-        crypt = None
+        verify_hash = None
     else:
-        def crypt(secret, salt):
-            # encrypt method deprecated in favor of hash in passlib 1.7
-            if hasattr(des_crypt, 'hash'):
-                return des_crypt.using(salt=salt).hash(secret)
-            else:
-                return des_crypt.encrypt(secret, salt=salt)
+        def verify_hash(secret, the_hash):
+            return hmac.compare_digest(crypt(secret, the_hash), the_hash)
+else:
+    def verify_hash(secret, the_hash):
+        ctx = CryptContext(schemes=['bcrypt', 'sha256_crypt',
+                                    'sha512_crypt', 'des_crypt'])
+        try:
+            return ctx.verify(secret, the_hash)
+        except ValueError:
+            return False
 
 
 def rpartition(s, sep):
