@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -14,26 +14,26 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import print_function
-
 from abc import ABCMeta, abstractmethod
 from base64 import b64decode, b64encode
 from hashlib import md5, sha1
+import hmac
 import os
 import re
 import sys
-import urllib2
-import urlparse
+import urllib.parse
+import urllib.request
 
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.web.api import IAuthenticator, IRequestHandler
 from trac.web.chrome import Chrome, INavigationContributor
 from trac.util import hex_entropy, md5crypt
-from trac.util.compat import crypt
+from trac.util.compat import verify_hash
 from trac.util.concurrency import threading
 from trac.util.datefmt import time_now
 from trac.util.html import tag
+from trac.util.text import exception_to_unicode
 from trac.util.translation import _, tag_
 
 
@@ -267,9 +267,9 @@ class LoginModule(Component):
         if referer:
             if not referer.startswith(('http://', 'https://')):
                 # Make URL absolute
-                scheme, host = urlparse.urlparse(req.base_url)[:2]
-                referer = urlparse.urlunparse((scheme, host, referer, None,
-                                               None, None))
+                scheme, host = urllib.parse.urlparse(req.base_url)[:2]
+                referer = urllib.parse.urlunparse((scheme, host, referer, None,
+                                                   None, None))
             pos = req.base_url.find(':')
             base_scheme = req.base_url[:pos]
             base_noscheme = req.base_url[pos:]
@@ -288,9 +288,7 @@ class LoginModule(Component):
         return req.args.get('referer') or req.get_header('Referer')
 
 
-class HTTPAuthentication(object):
-
-    __metaclass__ = ABCMeta
+class HTTPAuthentication(object, metaclass=ABCMeta):
 
     @abstractmethod
     def do_auth(self, environ, start_response):
@@ -317,14 +315,14 @@ class BasicAuthentication(PasswordFileAuthentication):
     def __init__(self, htpasswd, realm):
         # FIXME pass a logger
         self.realm = realm
-        self.crypt = crypt
+        self.verify = verify_hash
         self.hash = {}
         PasswordFileAuthentication.__init__(self, htpasswd)
 
     def load(self, filename):
         # FIXME use a logger
         self.hash = {}
-        with open(filename) as fd:
+        with open(filename, encoding='utf-8') as fd:
             for line in fd:
                 line = line.split('#')[0].strip()
                 if not line:
@@ -335,7 +333,7 @@ class BasicAuthentication(PasswordFileAuthentication):
                     print("Warning: invalid password line in %s: %s"
                           % (filename, line), file=sys.stderr)
                     continue
-                if '$' in h or h.startswith('{SHA}') or self.crypt:
+                if h.startswith(('$apr1$', '{SHA}')) or self.verify:
                     self.hash[u] = h
                 else:
                     print('Warning: cannot parse password for user "%s" '
@@ -352,28 +350,40 @@ class BasicAuthentication(PasswordFileAuthentication):
         if the_hash is None:
             return False
 
+        # SHA-1
         if the_hash.startswith('{SHA}'):
-            return b64encode(sha1(password).digest()) == the_hash[5:]
+            hash_ = str(b64encode(sha1(password.encode('utf-8')).digest()),
+                        'ascii')
+            return hmac.compare_digest(hash_, the_hash[5:])
 
-        if '$' not in the_hash:
-            return self.crypt(password, the_hash[:2]) == the_hash
+        # MD5
+        if the_hash.startswith('$apr1$'):
+            magic, salt = the_hash[1:].split('$')[:2]
+            magic = '$' + magic + '$'
+            return hmac.compare_digest(md5crypt(password, salt, magic),
+                                       the_hash)
 
-        magic, salt = the_hash[1:].split('$')[:2]
-        magic = '$' + magic + '$'
-        return md5crypt(password, salt, magic) == the_hash
+        # crypt compatible
+        try:
+            return self.verify(password, the_hash)
+        except RuntimeError as e:
+            print('Warning: cannot verify password for user "%s" (%s)' %
+                  (user, exception_to_unicode(e)), file=sys.stderr)
+            return False
 
     def do_auth(self, environ, start_response):
         header = environ.get('HTTP_AUTHORIZATION')
         if header and header.startswith('Basic'):
-            auth = b64decode(header[6:]).split(':')
+            auth = str(b64decode(header[6:]), 'utf-8').split(':', 1)
             if len(auth) == 2:
                 user, password = auth
                 if self.test(user, password):
                     return user
 
-        start_response('401 Unauthorized',
-                       [('WWW-Authenticate', 'Basic realm="%s"' % self.realm),
-                        ('Content-Length', '0')])('')
+        headers = [('WWW-Authenticate', 'Basic realm="%s"' % self.realm),
+                   ('Content-Length', '0')]
+        write = start_response('401 Unauthorized', headers)
+        write(b'')
 
 
 class DigestAuthentication(PasswordFileAuthentication):
@@ -395,7 +405,7 @@ class DigestAuthentication(PasswordFileAuthentication):
         """
         # FIXME use a logger
         self.hash = {}
-        with open(filename) as fd:
+        with open(filename, encoding='utf-8') as fd:
             for line in fd:
                 line = line.split('#')[0].strip()
                 if not line:
@@ -414,7 +424,7 @@ class DigestAuthentication(PasswordFileAuthentication):
 
     def parse_auth_header(self, authorization):
         values = {}
-        for value in urllib2.parse_http_list(authorization):
+        for value in urllib.request.parse_http_list(authorization):
             n, v = value.split('=', 1)
             if v[0] == '"' and v[-1] == '"':
                 values[n] = v[1:-1]
@@ -423,18 +433,19 @@ class DigestAuthentication(PasswordFileAuthentication):
         return values
 
     def send_auth_request(self, environ, start_response, stale='false'):
-        """Send a digest challange to the browser. Record used nonces
+        """Send a digest challenge to the browser. Record used nonces
         to avoid replay attacks.
         """
         nonce = hex_entropy()
         self.active_nonces.append(nonce)
         if len(self.active_nonces) > self.MAX_NONCES:
             self.active_nonces = self.active_nonces[-self.MAX_NONCES:]
-        start_response('401 Unauthorized',
-                       [('WWW-Authenticate',
-                        'Digest realm="%s", nonce="%s", qop="auth", stale="%s"'
-                        % (self.realm, nonce, stale)),
-                        ('Content-Length', '0')])('')
+        headers = [('WWW-Authenticate',
+                    'Digest realm="%s", nonce="%s", qop="auth", stale="%s"' %
+                    (self.realm, nonce, stale)),
+                   ('Content-Length', '0')]
+        write = start_response('401 Unauthorized', headers)
+        write(b'')
 
     def do_auth(self, environ, start_response):
         header = environ.get('HTTP_AUTHORIZATION')
@@ -456,7 +467,7 @@ class DigestAuthentication(PasswordFileAuthentication):
             self.send_auth_request(environ, start_response)
             return None
 
-        kd = lambda x: md5(':'.join(x)).hexdigest()
+        kd = lambda x: md5(b':'.join(v.encode('utf-8') for v in x)).hexdigest()
         a1 = self.hash[auth['username']]
         a2 = kd([environ['REQUEST_METHOD'], auth['uri']])
         # Is the response correct?
