@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
@@ -124,7 +124,7 @@ def parse_args(args=None):
 
         def __init__(self, option_strings, dest, nargs=None, **kwargs):
             self.cls = kwargs.pop('cls')
-            super(_AuthAction, self).__init__(option_strings, dest, nargs,
+            super().__init__(option_strings, dest, nargs,
                                               **kwargs)
 
         def __call__(self, parser, namespace, values, option_string=None):
@@ -150,7 +150,7 @@ def parse_args(args=None):
 
         def __init__(self, option_strings, dest, nargs=None, **kwargs):
             self.must_exist = kwargs.pop('must_exist', False)
-            super(_PathAction, self).__init__(option_strings, dest, nargs,
+            super().__init__(option_strings, dest, nargs,
                                               **kwargs)
 
         def __call__(self, parser, namespace, values, option_string=None):
@@ -344,6 +344,7 @@ def main():
                 httpd.socket = ssl.wrap_socket(httpd.socket, server_side=True,
                                                certfile=args.certfile,
                                                keyfile=args.keyfile)
+                httpd.environ['HTTPS'] = 'yes'
             httpd.serve_forever()
     elif args.protocol in ('scgi', 'ajp', 'fcgi'):
         def serve():
