@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -201,7 +201,7 @@ class IWikiSyntaxProvider(Interface):
         """
 
 def parse_args(args, strict=True):
-    """Utility for parsing macro "content" and splitting them into arguments.
+    r"""Utility for parsing macro "content" and splitting them into arguments.
 
     The content is split along commas, unless they are escaped with a
     backquote (see example below).
