@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2021 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
@@ -127,7 +127,7 @@ class ProcessorError(TracError):
 
 class WikiProcessor(object):
 
-    _code_block_re = re.compile('^<div(?:\s+class="([^"]+)")?>(.*)</div>$')
+    _code_block_re = re.compile(r'^<div(?:\s+class="([^"]+)")?>(.*)</div>$')
     _block_elem_re = re.compile(r'^\s*<(?:div|table)(?:\s+[^>]+)?>',
                                 re.I | re.M)
 
