@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2020 Edgewall Software
+# Copyright (C) 2007-2023 Edgewall Software
 # Copyright (C) 2007 Alec Thomas <alec@swapoff.org>
 # All rights reserved.
 #
@@ -14,12 +14,12 @@
 #
 # Author: Alec Thomas <alec@swapoff.org>
 
+import configparser
 import os
 from fnmatch import fnmatchcase
 from itertools import groupby
 
-from trac.config import ConfigurationError, ParsingError, PathOption, \
-                        UnicodeConfigParser
+from trac.config import ConfigurationError, PathOption, UnicodeConfigParser
 from trac.core import Component, implements
 from trac.perm import IPermissionPolicy, PermissionSystem
 from trac.util import to_list
@@ -64,7 +64,7 @@ class AuthzPolicy(Component):
 
        Resources are ordered left to right, from parent to child. If any
        component is inapplicable, `*` is substituted. If the version pattern is
-       not specified explicitely, all versions (`@*`) is added implicitly
+       not specified explicitly, all versions (`@*`) is added implicitly
 
        Example: Match the WikiStart page::
 
@@ -174,7 +174,7 @@ class AuthzPolicy(Component):
                            "option in trac.ini is empty or not defined.")
             raise ConfigurationError()
         try:
-            self.authz_mtime = os.path.getmtime(self.authz_file)
+            authz_mtime = os.path.getmtime(self.authz_file)
         except OSError as e:
             self.log.error("Error parsing authz permission policy file: %s",
                            exception_to_unicode(e))
@@ -183,7 +183,7 @@ class AuthzPolicy(Component):
         self.authz = UnicodeConfigParser(ignorecase_option=False)
         try:
             self.authz.read(self.authz_file)
-        except ParsingError as e:
+        except configparser.ParsingError as e:
             self.log.error("Error parsing authz permission policy file: %s",
                            exception_to_unicode(e))
             raise ConfigurationError()
@@ -209,7 +209,7 @@ class AuthzPolicy(Component):
         for section in self.authz.sections():
             if section == 'groups':
                 continue
-            for _, actions in self.authz.items(section):
+            for user, actions in self.authz.items(section):
                 for action in to_list(actions):
                     if action.startswith('!'):
                         action = action[1:]
@@ -217,6 +217,7 @@ class AuthzPolicy(Component):
                         self.log.warning("The action %s in the [%s] section "
                                          "of %s is not a valid action.",
                                          action, section, authz_basename)
+        self.authz_mtime = authz_mtime
 
     def normalise_resource(self, resource):
         def to_descriptor(resource):
