@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2020 Edgewall Software
+# Copyright (C) 2012-2023 Edgewall Software
 # Copyright (C) 2006-2011, Herbert Valerio Riedel <hvr@gnu.org>
 # All rights reserved.
 #
@@ -18,6 +18,7 @@ import io
 import os
 import re
 import subprocess
+import tempfile
 import weakref
 from collections import deque
 from functools import partial
@@ -25,10 +26,10 @@ from subprocess import DEVNULL, PIPE
 from threading import Lock
 
 from trac.core import TracBaseError
-from trac.util import terminate
+from trac.util import as_int, terminate
 from trac.util.compat import close_fds
 from trac.util.datefmt import time_now
-from trac.util.text import to_unicode
+from trac.util.text import exception_to_unicode, to_unicode
 
 __all__ = ['GitError', 'GitErrorSha', 'Storage', 'StorageFactory']
 
@@ -332,35 +333,31 @@ class Storage(object):
     def git_version(git_bin='git'):
         GIT_VERSION_MIN_REQUIRED = (1, 5, 6)
         try:
-            g = GitCore(git_bin=git_bin)
-            [v] = g.version().splitlines()
-            version = v.strip().split()[2]
-            # 'version' has usually at least 3 numeric version
-            # components, e.g.::
-            #  1.5.4.2
-            #  1.5.4.3.230.g2db511
-            #  1.5.4.GIT
-
-            def try_int(s):
-                try:
-                    return int(s)
-                except ValueError:
-                    return s
-
-            split_version = tuple(map(try_int, version.split(b'.')))
-
-            result = {}
-            result['v_str'] = version
-            result['v_tuple'] = split_version
-            result['v_min_tuple'] = GIT_VERSION_MIN_REQUIRED
-            result['v_min_str'] = ".".join(map(str, GIT_VERSION_MIN_REQUIRED))
-            result['v_compatible'] = split_version >= GIT_VERSION_MIN_REQUIRED
-            return result
-
-        except Exception as e:
-            raise GitError("Could not retrieve GIT version (tried to "
-                           "execute/parse '%s --version' but got %s)"
-                           % (git_bin, repr(e)))
+            version = GitCore(git_bin=git_bin).version()
+        except OSError as e:
+            raise GitError("Could not retrieve GIT version. '%s --version' "
+                           "(%s)" % (git_bin, exception_to_unicode(e)))
+        version = str(version, 'latin1')
+        # 'version' should have at least 3 numeric version components:
+        #  1.5.6
+        #  1.5.6.windows.1
+        #  1.5.6.2
+        #  1.5.6.3.230.g2db511
+        #  1.5.6.GIT
+        m = re.match('git version (.*)\n$', version)
+        if not m:
+            raise GitError("Could not retrieve GIT version. "
+                           "'%s --version' returned %s"
+                           % (git_bin, repr(version)))
+        version_str = m.group(1)
+        version_tuple = tuple(as_int(s, s) for s in version_str.split('.'))
+        return {
+            'v_str': version_str,
+            'v_tuple': version_tuple,
+            'v_min_str': '.'.join(map(str, GIT_VERSION_MIN_REQUIRED)),
+            'v_min_tuple': GIT_VERSION_MIN_REQUIRED,
+            'v_compatible': version_tuple >= GIT_VERSION_MIN_REQUIRED,
+        }
 
     def __init__(self, git_dir, log, git_bin='git', git_fs_encoding=None,
                  rev_cache=None):
@@ -705,6 +702,9 @@ class Storage(object):
         return self.verifyrev('HEAD')
 
     def cat_file(self, kind, sha):
+        return self._cat_file_reader(kind, sha).read()
+
+    def _cat_file_reader(self, kind, sha):
         with self.__cat_file_pipe_lock:
             if self.__cat_file_pipe is None:
                 self.__cat_file_pipe = self.repo.cat_file_batch()
@@ -726,12 +726,35 @@ class Storage(object):
                                    "kind %r, expected %r)" % (_type, kind))
 
                 size = int(_size)
-                return self.__cat_file_pipe.stdout.read(size + 1)[:size]
-            except EnvironmentError:
+
+                # stdout.read() can return fewer bytes than requested,
+                # especially if a pipe buffers because the contents are
+                # larger than 64k.
+                stdout_read = self.__cat_file_pipe.stdout.read
+                if size > 32 * 1024 * 1024:
+                    buf = tempfile.TemporaryFile()
+                else:
+                    buf = io.BytesIO()
+                remaining = size + 1
+                while remaining > 0:
+                    chunk = stdout_read(min(remaining, 65536))
+                    if not chunk:
+                        # No new data, let's abort
+                        raise GitError("internal error (expected to read %d "
+                                       "bytes, but only got %d)" %
+                                       (size + 1, size + 1 - remaining))
+                    remaining -= len(chunk)
+                    buf.write(chunk if remaining > 0 else chunk[:-1])
+
+                buf.seek(0)
+                return buf
+
+            except EnvironmentError as e:
                 # There was an error, we should close the pipe to get to a
                 # consistent state (Otherwise it happens that next time we
                 # call cat_file we get payload from previous call)
-                self.logger.debug("closing cat_file pipe")
+                self.logger.warning("closing cat_file pipe: %s",
+                                    exception_to_unicode(e))
                 self._cleanup_proc(self.__cat_file_pipe)
                 self.__cat_file_pipe = None
 
@@ -889,13 +912,12 @@ class Storage(object):
 
     def get_file(self, sha):
         sha = _rev_b(sha)
-        content = self.cat_file(b'blob', sha)
-        return io.BytesIO(content)
+        return self._cat_file_reader(b'blob', sha)
 
     def get_obj_size(self, sha):
         sha = _rev_b(sha)
         try:
-            obj_size = int(self.repo.cat_file('-s', sha).strip())
+            obj_size = int(self.repo.cat_file(b'-s', sha).strip())
         except ValueError:
             raise GitErrorSha("object '%s' not found" % sha)
         return obj_size
