@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2021 Edgewall Software
+# Copyright (C) 2005-2023 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -275,7 +275,7 @@ class SubversionRepository(Repository):
                   svn_error=exception_to_unicode(e)))
         self.fs_ptr = repos.svn_repos_fs(self.repos)
 
-        self.uuid = fs.get_uuid(self.fs_ptr, self.pool)
+        self.uuid = _from_svn(fs.get_uuid(self.fs_ptr, self.pool))
         self.base = 'svn:%s:%s' % (self.uuid, root_path)
         name = 'svn:%s:%s' % (self.uuid, path)
 
@@ -380,7 +380,7 @@ class SubversionRepository(Repository):
     def get_quickjump_entries(self, rev):
         """Retrieve known branches, as (name, id) pairs.
 
-        Purposedly ignores `rev` and always takes the last revision.
+        Purposely ignores `rev` and always takes the last revision.
         """
         for n in self._get_tags_or_branches('branches'):
             yield 'branches', n.path, n.path, None
@@ -509,7 +509,7 @@ class SubversionRepository(Repository):
         return self.youngest
 
     def previous_rev(self, rev, path=''):
-        """Return revision immediately preceeding `rev`, eventually below
+        """Return revision immediately preceding `rev`, eventually below
         given `path` or globally.
         """
         # FIXME optimize for non-scoped
@@ -827,7 +827,7 @@ class SubversionNode(Node):
     def _get_prop(self, name):
         value = fs.node_prop(self.root, self._scoped_path_utf8, name,
                              self.pool)
-        return to_unicode(value)
+        return _from_svn(value)
 
     def get_branch_origin(self):
         """Return the revision in which the node's path was created.
