@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2020 Edgewall Software
+# Copyright (C) 2004-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -136,7 +136,7 @@ class TracAdminTestCaseBase(unittest.TestCase):
         if '[...]' in expected_result:
             m = re.match('.*'.join(map(re.escape,
                                        expected_result.split('[...]'))) +
-                         '\Z',
+                         r'\Z',
                          output, re.DOTALL)
             self.assertTrue(m, msg)
         else:
