@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009-2020 Edgewall Software
+# Copyright (C) 2009-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -94,7 +94,7 @@ class CachedProperty(CachedPropertyBase):
     """
 
     def __init__(self, retriever, key_attr):
-        super(CachedProperty, self).__init__(retriever)
+        super().__init__(retriever)
         self.key_attr = key_attr
 
     def __get__(self, instance, owner):
@@ -115,7 +115,7 @@ class CachedProperty(CachedPropertyBase):
 
 
 def cached(fn_or_attr=None):
-    """Method decorator creating a cached attribute from a data
+    r"""Method decorator creating a cached attribute from a data
     retrieval method.
 
     Accessing the cached attribute gives back the cached value.  The
