@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
@@ -57,7 +57,7 @@ class PermissionError(TracBaseError):
                         "permissions.", perm=self.action)
         elif msg is None:
             msg = _("Insufficient privileges to perform this operation.")
-        super(PermissionError, self).__init__(msg)
+        super().__init__(msg)
 
     @property
     def message(self):
@@ -774,7 +774,7 @@ class PermissionAdmin(Component):
                 printout(e)
                 return action
             except TracError as e:
-                raise AdminCommandError(e)
+                raise AdminCommandError(e) from e
 
         # An exception rolls back the atomic transaction so it's
         # necessary to retry the transaction after removing the
