@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2021 Edgewall Software
+# Copyright (C) 2004-2023 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -1257,7 +1257,7 @@ class QueryModule(Component):
 class TicketQueryMacro(WikiMacroBase):
     _domain = 'messages'
     _description = cleandoc_(
-    """Wiki macro listing tickets that match certain criteria.
+    r"""Wiki macro listing tickets that match certain criteria.
 
     This macro accepts a comma-separated list of keyed parameters,
     in the form "key=value".
