@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2021 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -1347,7 +1347,7 @@ class TicketModule(Component):
         except Exception as e:
             self.log.error("Failure sending notification on creation of "
                            "ticket #%s: %s",
-                           ticket.id, exception_to_unicode(e))
+                           ticket.id, exception_to_unicode(e, traceback=True))
             add_warning(req, tag_("The ticket has been created, but an error "
                                   "occurred while sending notifications: "
                                   "%(message)s", message=to_fragment(e)))
@@ -1387,8 +1387,8 @@ class TicketModule(Component):
                 NotificationSystem(self.env).notify(event)
             except Exception as e:
                 self.log.error("Failure sending notification on change to "
-                               "ticket #%s: %s",
-                               ticket.id, exception_to_unicode(e))
+                               "ticket #%s: %s", ticket.id,
+                               exception_to_unicode(e, traceback=True))
                 # TRANSLATOR: The 'change' has been saved... (link)
                 change = _('change')
                 if fragment:
@@ -1822,7 +1822,7 @@ class TicketModule(Component):
                 diff = tag.a(_("diff"), href=href)
                 rendered = tag_("modified (%(diff)s)", diff=diff)
         elif type_ == 'text' and field_info.get('format') == 'list':
-            tl = functools.partial(to_list, sep='[,;\s]+')
+            tl = functools.partial(to_list, sep=r'[,;\s]+')
             rendered = render_list(None, tl, old, new)
         elif type_ == 'time':
             format_ = field_info.get('format')
