@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2021 Edgewall Software
+# Copyright (C) 2003-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -25,6 +25,10 @@ from html import entities
 from html.parser import HTMLParser
 
 from markupsafe import Markup, escape as escape_quotes
+try:
+    from markupsafe import soft_str as soft_unicode
+except ImportError:
+    from markupsafe import soft_unicode
 
 try:
     from babel.support import LazyProxy
@@ -89,6 +93,8 @@ def escape(text, quotes=True):
     :rtype: `Markup`
 
     """
+    if LazyProxy and isinstance(text, LazyProxy):
+        text = text.value
     if isinstance(text, Markup):
         return text
     if isinstance(text, Fragment):
@@ -603,7 +609,7 @@ class TracHTMLSanitizer(object):
     # IE6 <http://openmya.hacker.jp/hasegawa/security/expression.txt>
     #     7) Particular bit of Unicode characters
     _URL_FINDITER = re.compile(
-        '[Uu][Rr\u0280][Ll\u029F]\s*\(([^)]+)').finditer
+        '[Uu][Rr\u0280][Ll\u029F]\\s*\\(([^)]+)').finditer
 
     def sanitize(self, html):
         """Transforms the incoming HTML by removing anything's that deemed
@@ -964,6 +970,8 @@ def plaintext(text, keeplinebreaks=True):
     :param keeplinebreaks: optionally keep linebreaks
 
     """
+    if LazyProxy and isinstance(text, LazyProxy):
+        text = text.value
     if isinstance(text, Fragment):
         text = text.as_text()
     else:
@@ -978,6 +986,8 @@ def find_element(frag, attr=None, cls=None, tag=None):
     attribute, class or tag, using a preorder depth-first search.
 
     """
+    if LazyProxy and isinstance(frag, LazyProxy):
+        frag = frag.value
     if isinstance(frag, Element):
         if attr is not None and attr in frag.attrib:
             return frag
