@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2021 Edgewall Software
+# Copyright (C) 2005-2023 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Matthew Good <trac@matt-good.net>
 # All rights reserved.
@@ -46,7 +46,7 @@ class CGIGateway(WSGIGateway):
             sys.stdout.write('\r\n')
             sys.stdout.flush()
 
-        sys.stdout.write(data)
+        sys.stdout.buffer.write(data)
         sys.stdout.flush()
 
 
