@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2016-2020 Edgewall Software
+# Copyright (C) 2016-2023 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -154,7 +154,7 @@ StatementTuple = namedtuple('StatementTuple',
 
 class Statement(StatementTuple):
     def __new__(cls, *args, **kwargs):
-        self = super(Statement, cls).__new__(cls, *args, **kwargs)
+        self = super().__new__(cls, *args, **kwargs)
         self.is_block = (self.kw in JINJA2_BLOCK_KEYWORDS or
                          self.kw == 'set' and '=' not in self.expr)
         return self
