@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2020 Edgewall Software
+# Copyright (C) 2013-2023 Edgewall Software
 # Copyright (C) 2013 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -51,7 +51,7 @@ po_headers_re = re.compile(r'''
 
 
 def sanitize_file(path):
-    with open(path, 'r+') as f:
+    with open(path, 'r+', encoding='utf-8') as f:
         sanitized, nsub = ignore_lineno_re.subn(r'\1', f.read())
         sanitized, nsub2 = po_headers_re.subn(r'\1', sanitized)
         nsub += nsub2
